import logging


try:
    from . import hollow_controller as hollow_controller
except Exception:
    import n0s1.controllers.hollow_controller as hollow_controller


class AsanaController(hollow_controller.HollowController):
    def __init__(self):
        super().__init__()
        self._client = None

    def set_config(self, config):
        import asana
        TOKEN = config.get("token", "")
        self._client = asana.Client.access_token(TOKEN)
        return self.is_connected()

    def get_name(self):
        return "Asana"

    def is_connected(self):
        if self._client:
            if user := self._client.users.get_user("me"):
                self.log_message(f"Logged to {self.get_name()} as {user}")
            else:
                self.log_message(f"Unable to connect to {self.get_name()} instance. Check your credentials.", logging.ERROR)
                return False

            if spaces := self._client.workspaces.get_workspaces():
                workspace_found = False
                project_found = False
                for s in spaces:
                    workspace_gid = s.get("gid", "")
                    if len(workspace_gid) > 0:
                        workspace_found = True
                        if projects := self._client.projects.get_projects_for_workspace(workspace_gid):
                            for p in projects:
                                project_found = True
                                break
                if workspace_found:
                    if project_found:
                        return True
                    else:
                        self.log_message(f"Unable to list {self.get_name()} projects. Check your permissions.", logging.ERROR)
                else:
                    self.log_message(f"Unable to list {self.get_name()} workspaces. Check your permissions.", logging.ERROR)
            else:
                self.log_message(f"Unable to connect to {self.get_name()} instance. Check your credentials.", logging.ERROR)
        return False

    def get_data(self, include_coments=False, limit=None):
        if not self._client:
            return {}

        if workspaces := self._client.workspaces.get_workspaces():
            for w in workspaces:
                workspace_gid = w.get("gid", "")
                if projects := self._client.projects.get_projects_for_workspace(workspace_gid):
                    for p in projects:
                        project_gid = p.get("gid", "")
                        if tasks := self._client.tasks.get_tasks_for_project(project_gid, opt_fields=["name", "gid", "notes", "permalink_url"]):
                            for t in tasks:
                                comments = []
                                title = t.get("name", "")
                                task_gid = t.get("gid", "")
                                description = t.get("notes", "")
                                url = t.get("permalink_url", "")
                                if include_coments:
                                    if stories := self._client.stories.get_stories_for_task(task_gid):
                                        for s in stories:
                                            if s.get("type", "").lower() == "comment".lower():
                                                comment = s.get("text", "")
                                                comments.append(comment)
                                ticket = self.pack_data(title, description, comments, url, task_gid)
                                yield ticket

    def post_comment(self, task_gid, comment):
        if not self._client:
            return False
        if comment_status := self._client.stories.create_story_for_task(task_gid, {"type": "comment", "text": comment}):
            status = comment_status.get("text", "")
        return len(status) > 0



import requests
import json
import os

# Your integration token
NOTION_API_TOKEN = os.getenv("NOTION_TOKEN")
# Your database ID
DATABASE_ID = '591db2acac96450fafdb2112352d3d2d'


def query_database():
    url = f'https://api.notion.com/v1/databases/{DATABASE_ID}/query'
    headers = {
        'Authorization': f'Bearer {NOTION_API_TOKEN}',
        'Content-Type': 'application/json',
        'Notion-Version': '2022-06-28'
    }

    response = requests.post(url, headers=headers)

    if response.status_code == 200:
        return response.json()
    else:
        print(f"Error: {response.status_code}")
        print(response.text)
        return None


def list_tasks():
    data = query_database()
    if not data:
        return

    tasks = data.get('results', [])
    for task in tasks:
        properties = task.get('properties', {})
        name_property = properties.get('Name', {})
        name_text = name_property.get('title', [{}])[0].get('text', {}).get('content', 'No Title')
        status_property = properties.get('Status', {})
        status_text = status_property.get('select', {}).get('name', 'No Status')

        print(f"Task: {name_text} | Status: {status_text}")


if __name__ == "__main__":
    list_tasks()


import logging
import time

try:
    from . import hollow_controller as hollow_controller
except Exception:
    import n0s1.controllers.hollow_controller as hollow_controller


class JiraController(hollow_controller.HollowController):
    def __init__(self):
        super().__init__()
        self._client = None

    def set_config(self, config):
        from jira import JIRA
        SERVER = config.get("server", "")
        EMAIL = config.get("email", "")
        TOKEN = config.get("token", "")
        TIMEOUT = config.get("timeout", -1)
        VERIFY_SSL = not config.get("insecure", False)
        options = {"verify": VERIFY_SSL}
        if EMAIL and len(EMAIL) > 0:
            if TIMEOUT and TIMEOUT > 0:
                self._client = JIRA(SERVER, options=options, basic_auth=(EMAIL, TOKEN), timeout=TIMEOUT)
            else:
                self._client = JIRA(SERVER, options=options, basic_auth=(EMAIL, TOKEN))
        else:
            if TIMEOUT and TIMEOUT > 0:
                self._client = JIRA(SERVER, options=options, token_auth=TOKEN, timeout=TIMEOUT)
            else:
                self._client = JIRA(SERVER, options=options, token_auth=TOKEN)
        return self.is_connected()

    def get_name(self):
        return "Jira"

    def is_connected(self):
        if self._client:
            if user := self._client.myself():
                self.log_message(f"Logged to {self.get_name()} as {user}")
            else:
                self.log_message(f"Unable to connect to {self.get_name()} instance. Check your credentials.", logging.ERROR)
                return False

            if projects := self._client.projects():
                project_found = False
                issue_found = False
                for key in projects:
                    project_found = True
                    ql = f"project = '{key}'"
                    for issue in self._client.search_issues(ql):
                        if issue:
                            issue_found = True
                            return True
                if project_found:
                    if issue_found:
                        return True
                    else:
                        self.log_message(f"Unable to list {self.get_name()} issues. Check your permissions.", logging.ERROR)
                else:
                    self.log_message(f"Unable to list {self.get_name()} projects. Check your permissions.", logging.ERROR)
            else:
                self.log_message(f"Unable to connect to {self.get_name()} instance. Check your credentials.", logging.ERROR)
        return False

    def get_data(self, include_coments=False, limit=None):
        from jira.exceptions import JIRAError
        if not self._client:
            return {}
        start = 0
        if not limit or limit < 0:
            limit = 50
        try:
            projects = self._client.projects()
        except Exception as e:
            message = str(e) + f" client.projects()"
            self.log_message(message, logging.WARNING)
            projects = []

        for key in projects:
            ql = f"project = '{key}'"
            self.log_message(f"Scanning Jira project: [{key}]...")
            issues_finished = False
            issue_start = start
            while not issues_finished:
                try:
                    issues = self._client.search_issues(ql, startAt=issue_start, maxResults=limit)
                except JIRAError as e:
                    self.log_message(f"Error while searching issues on Jira project: [{key}]. Skipping...", logging.WARNING)
                    self.log_message(e)
                    issues = [{}]
                    break
                except Exception as e:
                    message = str(e) + f" client.search_issues({ql}, startAt={issue_start}, maxResults={limit})"
                    self.log_message(message, logging.WARNING)
                    issues = [{}]
                    time.sleep(1)
                    continue
                issue_start += limit
                issues_finished = len(issues) <= 0
                for issue in issues:
                    url = issue.self.split('/rest/api')[0] + "/browse/" + issue.key;
                    title = issue.fields.summary
                    description = issue.fields.description
                    comments = []
                    if include_coments:
                        try:
                            issue_comments = self._client.comments(issue.id)
                            comments.extend(c.body for c in issue_comments)
                        except Exception as e:
                            message = str(e) + f" client.comments({issue.id})"
                            self.log_message(message, logging.WARNING)
                            comments = []
                            time.sleep(1)

                    ticket = self.pack_data(title, description, comments, url, issue.key)
                    yield ticket

    def post_comment(self, issue, comment):
        if not self._client:
            return False
        comment = comment.replace("#", "0")
        comment_status = self._client.add_comment(issue, body=comment)
        status = comment_status.id
        return bool(status and len(status) > 0 and int(status) > 0)



try:
    from . import hollow_controller as hollow_controller
except Exception:
    import n0s1.controllers.hollow_controller as hollow_controller


class PlatformFactory:
    def __init__(self):
        self._creators = {}

    def register_platform(self, platform, creator):
        self._creators[platform] = creator

    def get_platform(self, platform):
        if creator := self._creators.get(platform):
            return creator()
        return hollow_controller.HollowController()


global factory
factory = PlatformFactory()

try:
    from . import jira_controller as jira_controller
    from . import confluence_controller as confluence_controller
    from . import linear_controller as linear_controller
    from . import asana_controller as asana_controller
    from . import wrike_controller as wrike_controller
    from . import slack_controller as slack_controller
except Exception:
    import n0s1.controllers.jira_controller as jira_controller
    import n0s1.controllers.confluence_controller as confluence_controller
    import n0s1.controllers.linear_controller as linear_controller
    import n0s1.controllers.asana_controller as asana_controller
    import n0s1.controllers.wrike_controller as wrike_controller
    import n0s1.controllers.slack_controller as slack_controller

factory.register_platform("", jira_controller.JiraController)
factory.register_platform("jira", jira_controller.JiraController)
factory.register_platform("jira_scan", jira_controller.JiraController)
factory.register_platform("confluence", confluence_controller.ConfluenceController)
factory.register_platform("confluence_scan", confluence_controller.ConfluenceController)
factory.register_platform("linear", linear_controller.LinearController)
factory.register_platform("linear_scan", linear_controller.LinearController)
factory.register_platform("asana", asana_controller.AsanaController)
factory.register_platform("asana_scan", asana_controller.AsanaController)
factory.register_platform("wrike", wrike_controller.WrikeController)
factory.register_platform("wrike_scan", wrike_controller.WrikeController)
factory.register_platform("slack", slack_controller.SlackController)
factory.register_platform("slack_scan", slack_controller.SlackController)



import html
import logging
import time


try:
    from . import hollow_controller as hollow_controller
except Exception:
    import n0s1.controllers.hollow_controller as hollow_controller


class ConfluenceController(hollow_controller.HollowController):
    def __init__(self):
        super().__init__()
        self._client = None
        self._url = None
        self._user = None
        self._password = None

    def set_config(self, config):
        from atlassian import Confluence
        SERVER = config.get("server", "")
        EMAIL = config.get("email", "")
        TOKEN = config.get("token", "")
        TIMEOUT = config.get("timeout", -1)
        VERIFY_SSL = not config.get("insecure", False)
        self._url = SERVER
        self._user = EMAIL
        self._password = TOKEN
        if EMAIL and len(EMAIL) > 0:
            if TIMEOUT and TIMEOUT > 0:
                self._client = Confluence(url=SERVER, verify_ssl=VERIFY_SSL, username=EMAIL, password=TOKEN, timeout=TIMEOUT)
            else:
                self._client = Confluence(url=SERVER, verify_ssl=VERIFY_SSL, username=EMAIL, password=TOKEN)
        else:
            if TIMEOUT and TIMEOUT > 0:
                self._client = Confluence(url=SERVER, verify_ssl=VERIFY_SSL, token=TOKEN, timeout=TIMEOUT)
            else:
                self._client = Confluence(url=SERVER, verify_ssl=VERIFY_SSL, token=TOKEN)
        return self.is_connected()

    def get_name(self):
        return "Confluence"

    def _get_request(self, url):
        from requests.auth import HTTPBasicAuth
        import requests
        response = None
        try:
            if self._user:
                auth = HTTPBasicAuth(self._user, self._password)
                headers = {
                    "Content-Type": "application/json"
                }
                response = requests.request(
                    "GET",
                    url,
                    headers=headers,
                    auth=auth
                )
            else:
                headers = {
                    "Content-Type": "application/json",
                    "Authorization": f"Bearer {self._password}"
                }
                response = requests.request(
                    "GET",
                    url,
                    headers=headers
                )
        except Exception as e:
            self.log_message(str(e))
        return response

    def get_current_user(self):
        url = f"{self._url}/rest/api/user/current"
        response = self._get_request(url)
        user = response.json() if response and response.status_code == 200 else None
        if user:
            user_type = user.get("type", "")
            if len(user_type) > 0:
                return user
        else:
            url = f"{self._url}/wiki/rest/api/user/current"
            response = self._get_request(url)
            if response and response.status_code == 200:
                user = response.json()
            if user:
                user_type = user.get("type", "")
                if len(user_type) > 0:
                    return user
        return None

    def is_connected(self):
        if self._client:
            if user := self.get_current_user():
                self.log_message(f"Logged to {self.get_name()} as {user}")
            else:
                self.log_message(f"Unable to connect to {self.get_name()} instance. Check your credentials.", logging.ERROR)
                return False

            if spaces := self._client.get_all_spaces():
                space_found = False
                page_found = False
                for s in spaces.get("results", []):
                    key = s.get("key", "")
                    if len(key) > 0:
                        space_found = True
                        pages = self._client.get_all_pages_from_space(key)
                        if len(pages) > 0:
                            page_found = True
                            break
                if space_found:
                    if page_found:
                        return True
                    else:
                        self.log_message(f"Unable to list {self.get_name()} pages. Check your permissions.", logging.ERROR)
                else:
                    self.log_message(f"Unable to list {self.get_name()} spaces. Check your permissions.", logging.ERROR)
            else:
                self.log_message(f"Unable to connect to {self.get_name()} instance. Check your credentials.", logging.ERROR)
        return False

    def get_data(self, include_coments=False, limit=None):
        from atlassian.confluence import ApiPermissionError
        if not self._client:
            return {}

        space_start = 0
        if not limit or limit < 0:
            limit = 50
        finished = False
        while not finished:
            try:
                res = self._client.get_all_spaces(start=space_start, limit=limit)
                spaces = res.get("results", [])
            except Exception as e:
                message = str(e) + f" get_all_spaces(start={space_start}, limit={limit})"
                self.log_message(message, logging.WARNING)
                spaces = [{}]
                time.sleep(1)
                continue
            space_start += limit

            for s in spaces:
                key = s.get("key", "")
                self.log_message(f"Scanning Confluence space: [{key}]...")
                if len(key) > 0:
                    pages_start = 0
                    pages_finished = False
                    while not pages_finished:
                        try:
                            pages = self._client.get_all_pages_from_space(key, start=pages_start, limit=limit)
                        except ApiPermissionError as e:
                            message = str(e) + f" get_all_pages_from_space({key}, start={pages_start}, limit={limit}). Skipping..."
                            self.log_message(message, logging.WARNING)
                            pages = [{}]
                            break
                        except Exception as e:
                            message = str(e) + f" get_all_pages_from_space({key}, start={pages_start}, limit={limit})"
                            self.log_message(message, logging.WARNING)
                            pages = [{}]
                            time.sleep(1)
                            continue
                        pages_start += limit

                        for p in pages:
                            comments = []
                            title = p.get("title", "")
                            page_id = p.get("id", "")
                            try:
                                body = self._client.get_page_by_id(page_id, expand="body.storage")
                            except Exception as e:
                                message = str(e) + f" get_page_by_id({page_id})"
                                self.log_message(message, logging.WARNING)
                                body = {}
                                time.sleep(1)
                                continue

                            description = body.get("body", {}).get("storage", {}).get("value", "")
                            url = body.get("_links", {}).get("base", "") + p.get("_links", {}).get("webui", "")
                            if len(page_id) > 0 and include_coments:
                                comments_start = 0
                                comments_finished = False
                                while not comments_finished:
                                    try:
                                        comments_response = self._client.get_page_comments(page_id, expand="body.storage", start=comments_start, limit=limit)
                                        comments_result = comments_response.get("results", [])
                                    except Exception as e:
                                        message = str(e) + f" get_page_comments({page_id}, expand=\"body.storage\", start={comments_start}, limit={limit})"
                                        self.log_message(message, logging.WARNING)
                                        comments_result = [{}]
                                        time.sleep(1)
                                        continue
                                    comments_start += limit

                                    for c in comments_result:
                                        comment = c.get("body", {}).get("storage", {}).get("value", "")
                                        comments.append(comment)

                                    if len(comments_result) <= 0:
                                        comments_finished = True

                            ticket = self.pack_data(title, description, comments, url, page_id)
                            yield ticket

                        if len(pages) <= 0:
                            pages_finished = True

            if len(spaces) <= 0:
                finished = True

    def post_comment(self, issue, comment):
        if not self._client:
            return False
        comment = comment.replace("#", "0")
        comment = html.escape(comment, quote=True)
        status = -1
        if comment_status := self._client.add_comment(issue, comment):
            status = comment_status.get("id", "")
        return int(status) > 0



import argparse
import hashlib
import logging
import json
import os
import pathlib
import re
import sys
import toml
import yaml
from datetime import datetime, timezone

try:
    import controllers.spark1 as spark1
except:
    import n0s1.controllers.spark1 as spark1

try:
    import controllers.platform_controller as platform_controller
except:
    import n0s1.controllers.platform_controller as platform_controller

try:
    import reporting.report_sarif as report_sarif
except:
    import n0s1.reporting.report_sarif as report_sarif

try:
    import reporting.report_gitlab as report_gitlab
except:
    import n0s1.reporting.report_gitlab as report_gitlab

global n0s1_version, report_json, report_file, cfg, DEBUG


def log_message(message, level=logging.INFO):
    global DEBUG
    debug_file = "n0s1_debug.log"

    if level == logging.NOTSET or level == logging.DEBUG:
        logging.debug(message)
    if level == logging.INFO:
        logging.info(message)
    if level == logging.WARNING:
        logging.warning(message)
    if level == logging.ERROR:
        logging.error(message)
    if level == logging.CRITICAL:
        logging.critical(message)

    if DEBUG:
        with open(debug_file, "a") as f:
            if f.tell() == 0:
                f.write("Debug logging message for n0s1\n")
            f.write(f"{message}\n")


def init_argparse() -> argparse.ArgumentParser:
    global n0s1_version
    """Adds arguements that can be called from the command line

    Returns:
        argparse.ArgumentParser: A parser that contains the command line options
    """
    install_path = os.path.dirname(os.path.abspath(__file__))
    parser = argparse.ArgumentParser(
        prog="n0s1",
        description="""Secret scanner for Slack, Jira, Confluence, Asana, Wrike and Linear.
        """,
    )

    try:
        here = pathlib.Path(__file__).parent.resolve()
        init_file = pathlib.Path(here / "__init__.py")
        n0s1_version = re.search(r'^__version__ = [\'"]([^\'"]*)[\'"]', init_file.read_text(), re.M).group(1)
    except Exception:
        n0s1_version = "0.0.1"
    parser.add_argument('-v', '--version', action='version', version='%(prog)s ' + n0s1_version)

    # Create parent subparser. Note `add_help=False` and creation via `argparse.`
    parent_parser = argparse.ArgumentParser(add_help=False)
    parent_parser.add_argument(
        "--regex-file",
        dest="regex_file",
        nargs="?",
        default=f"{install_path}/config/regex.yaml",
        type=str,
        help="Custom .yaml or .toml with a list of regexes to be matched."
    )
    parent_parser.add_argument(
        "--config-file",
        dest="config_file",
        nargs="?",
        default=f"{install_path}/config/config.yaml",
        type=str,
        help="Configuration file (YAML format) to be used."
    )
    parent_parser.add_argument(
        "--report-file",
        dest="report_file",
        nargs="?",
        default="n0s1_report.json",
        type=str,
        help="Output report file for the leaked secrets."
    )
    parent_parser.add_argument(
        "--report-format",
        dest="report_format",
        nargs="?",
        default="n0s1",
        type=str,
        help="Output report format. Supported formats: n0s1, SARIF, gitlab."
    )
    parent_parser.add_argument(
        "--post-comment",
        dest="post_comment",
        action="store_true",
        help="By default, scans only flag leaked secrets; this adds a warning comment to every ticket with a potential secret leak",
    )
    parent_parser.add_argument(
        "--skip-comment",
        dest="skip_comment",
        action="store_true",
        help="By default, scans check the ticket title, description and comments; this flag disables ticket comment scanning",
    )
    parent_parser.add_argument(
        "--show-matched-secret-on-logs",
        dest="show_matched_secret_on_logs",
        action="store_true",
        help="By default, only a sanitized version of the leak is shown on logs. This flag makes the actual leaked secret to be displayed on logs. Be extra careful when enabling this flag because you might make the leak worst by sending sensitive info to logs.",
    )
    parent_parser.add_argument(
        "--debug",
        dest="debug",
        action="store_true",
        help="Debug mode. Warning it may further expose sensitive data.",
    )
    parent_parser.add_argument(
        "--secret-manager",
        dest="secret_manager",
        nargs="?",
        default="a secret manager tool",
        type=str,
        help="Secret manager tool name to be suggested when leaks are found."
    )
    parent_parser.add_argument(
        "--contact-help",
        dest="contact_help",
        nargs="?",
        default="contact@spark1.us",
        type=str,
        help="Contact information for assistance when leaks are detected."
    )
    parent_parser.add_argument(
        "--label",
        dest="label",
        nargs="?",
        default="n0s1bot_auto_comment_e869dd5fa15ca0749a350aac758c7f56f56ad9be1",
        type=str,
        help="Unique identifier to be added to the comments so that the n0s1 bot can recognize if the leak has been previously flagged."
    )
    parent_parser.add_argument(
        "--timeout",
        dest="timeout",
        nargs="?",
        default="",
        type=str,
        help="HTTP request timeout in seconds"
    )
    parent_parser.add_argument(
        "--limit",
        dest="limit",
        nargs="?",
        default="",
        type=str,
        help="The limit of the number of pages to return per HTTP request"
    )
    parent_parser.add_argument(
        "--insecure",
        dest="insecure",
        action="store_true",
        help="Insecure mode. Ignore SSL certificate verification",
    )
    subparsers = parser.add_subparsers(
        help="Subcommands", dest="command", metavar="COMMAND"
    )

    slack_scan_parser = subparsers.add_parser(
        "slack_scan", help="Scan Slack messages", parents=[parent_parser]
    )
    slack_scan_parser.add_argument(
        "--api-key",
        dest="api_key",
        nargs="?",
        type=str,
        help="Slack token with OAuth scope: search:read, users:read, chat:write. Ref: https://api.slack.com/tutorials/tracks/getting-a-token"
    )

    asana_scan_parser = subparsers.add_parser(
        "asana_scan", help="Scan Asana tasks", parents=[parent_parser]
    )
    asana_scan_parser.add_argument(
        "--api-key",
        dest="api_key",
        nargs="?",
        type=str,
        help="Asana API key. Ref: https://developers.asana.com/docs/personal-access-token#generating-a-pat"
    )

    wrike_scan_parser = subparsers.add_parser(
        "wrike_scan", help="Scan Wrike tasks", parents=[parent_parser]
    )
    wrike_scan_parser.add_argument(
        "--api-key",
        dest="api_key",
        nargs="?",
        type=str,
        help="Wrike permanent token. Ref: https://help.wrike.com/hc/en-us/articles/210409445-Wrike-API#UUID-a1b0051a-0537-2215-c542-3b04d7205f4b_section-idm232163770698441"
    )

    linear_scan_parser = subparsers.add_parser(
        "linear_scan", help="Scan Linear issues", parents=[parent_parser]
    )
    linear_scan_parser.add_argument(
        "--api-key",
        dest="api_key",
        nargs="?",
        type=str,
        help="Linear API key. Ref: https://developers.linear.app/docs/graphql/working-with-the-graphql-api#personal-api-keys"
    )

    jira_scan_parser = subparsers.add_parser(
        "jira_scan", help="Scan Jira tickets", parents=[parent_parser]
    )
    jira_scan_parser.add_argument(
        "--server",
        dest="server",
        nargs="?",
        type=str,
        help="Jira server uri."
    )
    jira_scan_parser.add_argument(
        "--email",
        dest="email",
        nargs="?",
        type=str,
        help="Jira user email."
    )
    jira_scan_parser.add_argument(
        "--api-key",
        dest="api_key",
        nargs="?",
        type=str,
        help="Jira API key."
    )

    confluence_scan_parser = subparsers.add_parser(
        "confluence_scan", help="Scan Confluence pages", parents=[parent_parser]
    )
    confluence_scan_parser.add_argument(
        "--server",
        dest="server",
        nargs="?",
        type=str,
        help="Confluence base URL (e.g. https://yourcompany.atlassian.net)."
    )
    confluence_scan_parser.add_argument(
        "--email",
        dest="email",
        nargs="?",
        type=str,
        help="Confluence user email."
    )
    confluence_scan_parser.add_argument(
        "--api-key",
        dest="api_key",
        nargs="?",
        type=str,
        help="Confluence API key."
    )
    return parser


def _sanitize_text(text, begin, end):
    text_len = len(text)
    s_begin = max(begin - 20, 0)
    s_end = min(end + 20, text_len)
    sanitized_text = f"{text[s_begin:begin]}<REDACTED>{text[end:s_end]}"
    snippet_text = text[s_begin:s_end]
    return sanitized_text, snippet_text


def _sha1_hash(to_hash):
    try:
        message_digest = hashlib.sha256()
        string_m = str(to_hash)
        byte_m = bytes(string_m, encoding='utf')
        message_digest.update(byte_m)
        return message_digest.hexdigest()
    except TypeError as e:
        raise "Unable to generate SHA-256 hash for input string" from e


def _save_report(report_format=""):
    global report_json, report_file

    try:
        if report_format.lower().find("sarif".lower()) != -1:
            github_report = report_sarif.n0s1_report_to_sarif_report(report_json)
            github_report.write_report(report_file)
            return True
        elif report_format.lower().find("gitlab".lower()) != -1:
            gitlab_report = report_gitlab.n0s1_report_to_gitlab_report(report_json)
            gitlab_report.write_report(report_file)
            return True
        else:
            with open(report_file, "w") as f:
                json.dump(report_json, f)
                return True
    except Exception as e:
        log_message(str(e), level=logging.ERROR)

    return False


def _safe_re_search(regex_str, text):
    global DEBUG
    m = None
    try:
        m = re.search(regex_str, text)
    except Exception:
        try:
            regex_str = regex_str.replace("(?i)", "")
            m = re.search(regex_str, text, re.IGNORECASE)
        except Exception as e:
            if DEBUG:
                log_message(str(e))
    return m


def match_regex(regex_config, text):
    for c in regex_config["rules"]:
        regex_str = c["regex"]
        if m := _safe_re_search(regex_str, text):
            begin = m.regs[0][0]
            end = m.regs[0][1]
            matched_text = text[begin:end]
            sanitized_text, snippet_text = _sanitize_text(text, begin, end)
            return c, matched_text, sanitized_text, snippet_text
    return None, None, None, None


def report_leaked_secret(scan_text_result, controller):
    global report_json, cfg
    snippet_text = scan_text_result.get("snippet_text", "")
    sanitized_secret = scan_text_result.get("sanitized_secret", "")
    matched = scan_text_result.get("matched_regex_config", {})
    regex_id = matched.get("id", "")
    regex_description = matched.get("description", "")
    regex = matched.get("regex", "")
    platform = scan_text_result.get("ticket_data", {}).get("platform", "")
    field = scan_text_result.get("ticket_data", {}).get("field", "")
    url = scan_text_result.get("ticket_data", {}).get("url", "")
    issue_id = scan_text_result.get("ticket_data", {}).get("issue_id", "")
    post_comment = scan_text_result.get("scan_arguments", {}).get("post_comment", False)
    show_matched_secret_on_logs = scan_text_result.get("scan_arguments", {}).get("show_matched_secret_on_logs", False)

    finding_info = "Platform:[{platform}] Field:[{field}] ID:[{regex_config_id}] Description:[{regex_config_description}] Regex: {regex}\n############## Sanitized Secret Leak ##############\n {leak}\n############## Sanitized Secret Leak ##############"
    finding_info = finding_info.format(regex_config_id=regex_id, regex_config_description=regex_description,
                                       regex=regex, platform=platform, field=field, leak=sanitized_secret)

    log_message("\nPotential secret leak regex match!")
    log_message(finding_info)
    if show_matched_secret_on_logs:
        log_message(
            f"\n##################### Secret  #####################\n{snippet_text}\n##################### Secret  #####################")
    log_message(f"\nLeak source: {url}")
    log_message("\n\n")
    finding_id = f"{url}_{sanitized_secret}"
    finding_id = _sha1_hash(finding_id)
    new_finding = {"id": finding_id, "url": url, "secret": sanitized_secret,
                   "details": {"matched_regex_config": scan_text_result["matched_regex_config"], "platform": platform,
                               "ticket_field": field}}
    if finding_id not in report_json["findings"]:
        report_json["findings"][finding_id] = new_finding
    if post_comment:
        comment_template = cfg.get("comment_params", {}).get("message_template", "")
        bot_name = cfg.get("comment_params", {}).get("bot_name", "bot")
        secret_manager = scan_text_result.get("scan_arguments", {}).get("secret_manager", "")
        contact_help = scan_text_result.get("scan_arguments", {}).get("contact_help", "")
        label = scan_text_result.get("scan_arguments", {}).get("label", "")
        format_variables = ["finding_info", "bot_name", "secret_manager", "contact_help", "label"]
        for variable in format_variables:
            if comment_template.find(variable) == -1:
                comment_template += f"\n{variable}: {{{variable}}}"
        comment = comment_template.format(finding_info=finding_info, bot_name=bot_name, secret_manager=secret_manager,
                                          contact_help=contact_help, label=label)
        if controller.get_name().lower() == "Slack".lower():
            comment = comment + f"\nLeak source: {url}"

        return controller.post_comment(issue_id, comment)
    return True


def scan_text(regex_config, text):
    global DEBUG
    match = False
    scan_text_result = {}
    try:
        matched_regex_config, secret, sanitized_secret, snippet_text = match_regex(regex_config, str(text))
        scan_text_result = {"matched_regex_config": matched_regex_config, "secret": secret,
                            "sanitized_secret": sanitized_secret, "snippet_text": snippet_text}
        if matched_regex_config:
            match = True
    except Exception as e:
        if DEBUG:
            log_message(str(e), level=logging.WARNING)
    return match, scan_text_result


def scan(regex_config, controller, scan_arguments):
    global DEBUG
    if not regex_config or not controller:
        return
    scan_comment = scan_arguments.get("scan_comment", False)
    post_comment = scan_arguments.get("post_comment", False)
    limit = scan_arguments.get("limit", None)

    for ticket in controller.get_data(scan_comment, limit):
        issue_id = ticket.get("issue_id")
        url = ticket.get("url")
        if DEBUG:
            log_message(f"Scanning [{issue_id}]: {url}")

        comments = ticket.get("ticket", {}).get("comments", {}).get("data", [])
        label = cfg.get("comment_params", {}).get("label", "")
        post_comment_for_this_issue = post_comment
        if post_comment_for_this_issue:
            for comment in comments:
                if comment.lower().find(label.lower()) != -1:
                    # Comment with leak warning has been already posted. Skip
                    # posting a new comment again
                    post_comment_for_this_issue = False
                    break
        scan_arguments["post_comment"] = post_comment_for_this_issue

        for key in ticket.get("ticket", {}):
            item = ticket.get("ticket", {}).get(key, {})
            name = item.get("name", "")
            data = item.get("data", None)
            data_type = item.get("data_type", None)
            if data_type and data_type.lower() == "str".lower():
                if data and data.lower().find(label.lower()) == -1:
                    scan_text_and_report_leaks(controller, data, name, regex_config, scan_arguments, ticket)
            elif data_type:
                for item_data in data:
                    if item_data and item_data.lower().find(label.lower()) == -1:
                        scan_text_and_report_leaks(controller, item_data, name, regex_config, scan_arguments, ticket)


def scan_text_and_report_leaks(controller, data, name, regex_config, scan_arguments, ticket):
    secret_found, scan_text_result = scan_text(regex_config, data)
    scan_text_result["ticket_data"] = ticket
    scan_text_result["ticket_data"]["field"] = name
    scan_text_result["ticket_data"]["platform"] = controller.get_name()
    scan_text_result["scan_arguments"] = scan_arguments
    if secret_found:
        report_leaked_secret(scan_text_result, controller)


def main(callback=None):
    global n0s1_version, report_json, report_file, cfg, DEBUG

    logging.basicConfig(level=logging.INFO)
    parser = init_argparse()
    args = parser.parse_args()

    regex_config = None
    cfg = {}

    if not args.command:
        parser.print_help()
        return

    if os.path.exists(args.regex_file):
        with open(args.regex_file, "r") as f:
            extension = os.path.splitext(args.regex_file)[1]
            if extension.lower() == ".yaml".lower():
                regex_config = yaml.load(f, Loader=yaml.FullLoader)
            else:
                regex_config = toml.load(f)
    else:
        log_message(f"Regex file [{args.regex_file}] not found!", level=logging.WARNING)

    if os.path.exists(args.config_file):
        with open(args.config_file, "r") as f:
            cfg = yaml.load(f, Loader=yaml.FullLoader)
    else:
        log_message(f"Config file [{args.config_file}] not found!", level=logging.WARNING)

    datetime_now_obj = datetime.now(timezone.utc)
    date_utc = datetime_now_obj.strftime("%Y-%m-%dT%H:%M:%S")

    report_json = {"tool": {"name": "n0s1", "version": n0s1_version, "author": "Spark 1 Security"},
                   "scan_date": {"timestamp": datetime_now_obj.timestamp(), "date_utc": date_utc},
                   "regex_config": regex_config, "findings": {}}
    report_file = args.report_file

    command = args.command
    controller_factory = platform_controller.factory
    controller = controller_factory.get_platform(command)
    controller.set_log_message_callback(callback)

    controller_config = {}

    if args.timeout and len(args.timeout) > 0:
        timeout = int(args.timeout)
    else:
        timeout = cfg.get("general_params", {}).get("timeout", None)

    if args.limit and len(args.limit) > 0:
        limit = int(args.limit)
    else:
        limit = cfg.get("general_params", {}).get("limit", None)

    if args.insecure:
        insecure = bool(args.insecure)
    else:
        insecure = cfg.get("general_params", {}).get("insecure", False)

    controller_config["timeout"] = timeout
    controller_config["limit"] = limit
    controller_config["insecure"] = insecure

    TOKEN = None
    SERVER = None
    EMAIL = None
    if command == "linear_scan":
        TOKEN = os.getenv("LINEAR_TOKEN")
        if args.api_key and len(args.api_key) > 0:
            TOKEN = args.api_key
        controller_config["token"] = TOKEN

    elif command == "slack_scan":
        TOKEN = os.getenv("SLACK_TOKEN")
        if args.api_key and len(args.api_key) > 0:
            TOKEN = args.api_key
        controller_config["token"] = TOKEN

    elif command == "asana_scan":
        TOKEN = os.getenv("ASANA_TOKEN")
        if args.api_key and len(args.api_key) > 0:
            TOKEN = args.api_key
        controller_config["token"] = TOKEN

    elif command == "wrike_scan":
        TOKEN = os.getenv("WRIKE_TOKEN")
        if args.api_key and len(args.api_key) > 0:
            TOKEN = args.api_key
        controller_config["token"] = TOKEN

    elif command == "jira_scan":
        SERVER = os.getenv("JIRA_SERVER")
        EMAIL = os.getenv("JIRA_EMAIL")
        TOKEN = os.getenv("JIRA_TOKEN")
        if args.server and len(args.server) > 0:
            SERVER = args.server
        if args.email and len(args.email) > 0:
            EMAIL = args.email
        if args.api_key and len(args.api_key) > 0:
            TOKEN = args.api_key
        controller_config["server"] = SERVER
        controller_config["email"] = EMAIL
        controller_config["token"] = TOKEN

    elif command == "confluence_scan":
        SERVER = os.getenv("CONFLUENCE_SERVER")
        if not SERVER:
            SERVER = os.getenv("JIRA_SERVER")
        EMAIL = os.getenv("CONFLUENCE_EMAIL")
        if not EMAIL:
            EMAIL = os.getenv("JIRA_EMAIL")
        TOKEN = os.getenv("CONFLUENCE_TOKEN")
        if not TOKEN:
            TOKEN = os.getenv("JIRA_TOKEN")
        if args.server and len(args.server) > 0:
            SERVER = args.server
        if args.email and len(args.email) > 0:
            EMAIL = args.email
        if args.api_key and len(args.api_key) > 0:
            TOKEN = args.api_key
        controller_config["server"] = SERVER
        controller_config["email"] = EMAIL
        controller_config["token"] = TOKEN
    else:
        parser.print_help()
        return

    DEBUG = args.debug

    message = f"n0s1 secret scanner version [{n0s1_version}] - Scan date: {date_utc}"
    log_message(message)
    if DEBUG:
        message = f"Args: {args}"
        log_message(message)
        message = f"Controller settings: {SERVER} {EMAIL}"
        if args.show_matched_secret_on_logs:
            message += f" {TOKEN}"
        log_message(message)

    if not controller.set_config(controller_config):
        sys.exit(-1)

    if args.post_comment:
        post_comment = args.post_comment
    else:
        post_comment = cfg.get("general_params", {}).get("post_comment", False)

    if args.skip_comment:
        skip_comment = args.skip_comment
    else:
        skip_comment = cfg.get("general_params", {}).get("skip_comment", False)
    scan_comment = not skip_comment

    if args.secret_manager:
        secret_manager = args.secret_manager
    else:
        secret_manager = cfg.get("comment_params", {}).get("secret_manager", False)

    if args.contact_help:
        contact_help = args.contact_help
    else:
        contact_help = cfg.get("comment_params", {}).get("contact_help", False)

    if args.label:
        label = args.label
    else:
        label = cfg.get("comment_params", {}).get("label", False)

    if args.show_matched_secret_on_logs:
        show_matched_secret_on_logs = args.show_matched_secret_on_logs
    else:
        show_matched_secret_on_logs = cfg.get("general_params", {}).get("show_matched_secret_on_logs", False)

    if args.report_format:
        report_format = args.report_format
    else:
        report_format = cfg.get("general_params", {}).get("report_format", "n0s1")

    scan_arguments = {"scan_comment": scan_comment, "post_comment": post_comment, "secret_manager": secret_manager,
                      "contact_help": contact_help, "label": label, "report_format": report_format, "debug": DEBUG,
                      "show_matched_secret_on_logs": show_matched_secret_on_logs, "scan_target": command,
                      "timeout": timeout, "limit": limit}
    report_json["tool"]["scan_arguments"] = scan_arguments

    N0S1_TOKEN = os.getenv("N0S1_TOKEN")
    n0s1_pro = spark1.Spark1(token_auth=N0S1_TOKEN)
    mode = "community"
    if n0s1_pro.is_connected(scan_arguments):
        mode = "professional"
    message = f"Starting scan in {mode} mode..."
    log_message(message)

    try:
        scan(regex_config, controller, scan_arguments)
    except KeyboardInterrupt:
        log_message("Keyboard interrupt detected. Saving findings and exiting...")
        sys.exit(130)
    except Exception as e:
        log_message("Execution interrupted by an exception. Saving partial report and exiting...")
        log_message(e)
        sys.exit(1)
    finally:
        _save_report(report_format)
        log_message("Done!")


if __name__ == "__main__":
    main(log_message)



import requests
from requests.models import Response
from types import ModuleType


class HttpClient:
    def __init__(
            self,
            headers: dict,
            logging: ModuleType,
            uri: str = None,
    ) -> None:
        if uri is None:
            raise Exception("Must specify URI")
        self.uri = uri
        self.headers = headers
        self.logging = logging

    def _delete_request(self, url: str = None, params: dict = None, headers: dict = None, data=None) -> Response:
        if url is None or len(url) <= 0:
            url = self.uri
        if headers:
            response = requests.delete(
                url,
                params=params,
                headers={**headers, **self.headers},
                data=data,
            )
        else:
            response = requests.delete(
                url,
                params=params,
                headers=self.headers,
                data=data,
            )
        self.logging.debug(
            f"HTTP DELETE request status: [{response.status_code}]. URL: {url}"
        )
        self.logging.debug(response.text)
        return response

    def _get_request(self, url: str = None, params: dict = None, headers: dict = None, data=None,
                     timeout=None) -> Response:
        if url is None or len(url) <= 0:
            url = self.uri
        if headers:
            response = requests.get(
                url,
                params=params,
                headers={**headers, **self.headers},
                data=data,
                timeout=timeout,
            )
        else:
            response = requests.get(
                url,
                params=params,
                headers=self.headers,
                data=data,
                timeout=timeout,
            )
        self.logging.debug(
            f"HTTP GET request status: [{response.status_code}]. URL: {url}"
        )
        self.logging.debug(response.text)
        return response

    def _post_request(
            self, url: str = None, params: dict = None, headers: dict = None, data=None, json: dict = None
    ) -> Response:
        if url is None or len(url) <= 0:
            url = self.uri
        if headers:
            response = requests.post(
                url,
                params=params,
                headers={**headers, **self.headers},
                data=data,
                json=json,
            )
        else:
            response = requests.post(
                url,
                params=params,
                headers=self.headers,
                data=data,
                json=json,
            )
        self.logging.debug(response.text)
        return response

    def _put_request(self, url: str = None, params: dict = None, headers: dict = None, data=None) -> Response:
        if url is None or len(url) <= 0:
            url = self.uri
        if headers:
            response = requests.put(
                url,
                params=params,
                headers={**headers, **self.headers},
                data=data,
            )
        else:
            response = requests.put(
                url,
                params=params,
                headers=self.headers,
                data=data,
            )
        self.logging.debug(response.text)
        return response



import logging
import time
from datetime import datetime, timezone
from types import ModuleType

try:
    import clients.http_client as http_client
except Exception:
    import n0s1.clients.http_client as http_client


def _generate_query_issues_with_pagination(pagination_arguments):
    query_issues_pagination = f"""
    {{
        issues{pagination_arguments} {{
            edges {{
                node {{
                    id
                    identifier
                    url
                    title
                    description
                    comments {{
                        nodes {{
                            id
                            body
                        }}
                    }}
                }}
                cursor
            }}
            pageInfo {{
                hasNextPage
                endCursor
            }}
        }}
    }}
    """
    return {"query": query_issues_pagination, "variables": {}}


class LinearGraphQLClient(http_client.HttpClient):
    def __init__(
            self,
            headers: dict,
            logging: ModuleType,
            uri: str = None,
    ) -> None:
        if not uri:
            uri = "https://api.linear.app"
        super().__init__(headers, logging, uri)
        self._user = None

    def graphql_query(self, query):
        url = f"{self.uri}/graphql/"
        r = self._post_request(url, json=query)

        # Check for rate limiting
        # https://developers.linear.app/docs/graphql/working-with-the-graphql-api/rate-limiting
        if r.status_code == 200 or 400 <= r.status_code < 500:
            rate_limit = r.headers.get("X-RateLimit-Requests-Limit", -1)
            rate_limit_remaining = r.headers.get("X-RateLimit-Requests-Remaining", -1)
            rate_limit_reset = r.headers.get("X-RateLimit-Requests-Reset", -1)
            if int(rate_limit_remaining) < 20:
                try:
                    self._extracted_from_graphql_query_13(
                        rate_limit_reset, rate_limit_remaining, rate_limit
                    )
                except Exception as e:
                    logging.warning(e)
        return r

    # TODO Rename this here and in `graphql_query`
    def _extracted_from_graphql_query_13(self, rate_limit_reset, rate_limit_remaining, rate_limit):
        timestamp_reset = float(rate_limit_reset) / 1000
        datetime_now_obj = datetime.now(timezone.utc)
        timestamp_now = datetime_now_obj.timestamp()
        retry_after = int(timestamp_reset - timestamp_now) + 5

        reset_datatime = datetime.utcfromtimestamp(timestamp_reset)
        self.logging.warning(
            f"Approaching rate limit! There are [{rate_limit_remaining}] requests remaining out of [{rate_limit}]. Current date: [{datetime_now_obj}] Rate Limit reset time: [{reset_datatime} UTC]. Retrying after [{retry_after}] seconds..."
        )
        if retry_after < 7200:
            time.sleep(retry_after)
        else:
            logging.warning(
                f"Retry after period is too long: [{retry_after}]. Skipping retry period. Header X-RateLimit-Requests-Reset set to {rate_limit_reset} and UTC epoch seconds now is {timestamp_now}.")

    def get_curret_user(self):
        query_me = f"""
        query Me {{
          viewer {{
            id
            name
            email
          }}
        }}
        """
        query = {"query": query_me}
        response = self.graphql_query(query)
        return response.json() if response.status_code == 200 else None

    def get_issue(self, id):
        query_issue = f"""
        query Issue {{
          issue(id: "{id}") {{
            id
            identifier
            url
            title
            description
            comments {{
              nodes {{
                id
                body
              }}
            }}
          }}
        }}
        """
        query = {"query": query_issue}
        response = self.graphql_query(query)
        return response.json() if response.status_code == 200 else None

    def set_issue_title(self, issue_id, title):
        query_issue = f"""
        mutation IssueUpdate {{
          issueUpdate(
            id: "{issue_id}",
            input: {{
              title: "{title}"
            }}
          ) {{
            success
            issue {{
              id
              title
              state {{
                id
                name
              }}
            }}
          }}
        }}
        """
        query = {"query": query_issue}
        response = self.graphql_query(query)
        return response.json() if response.status_code == 200 else None

    def add_comment(self, issue_id, comment):
        comment = comment.replace("\n", "\\n")
        query_issue = f"""
        mutation commentCreate {{
          commentCreate(
            input: {{
              issueId: "{issue_id}",
              body: "{comment}"
            }}
          )
          {{
            success
            comment {{
              id
            }}
          }}
        }}
        """
        query = {"query": query_issue}
        response = self.graphql_query(query)
        return response.json() if response.status_code == 200 else None

    def get_issues_and_comments(self, issues_per_page=100):
        pagination_arguments = f"(first: {issues_per_page})"
        query = _generate_query_issues_with_pagination(pagination_arguments)
        response = self.graphql_query(query)

        page_num = 0
        has_next_page = False
        cursor = None

        if response.status_code == 200:
            r = response.json()
            yield r
            has_next_page = r.get("data", {}).get("issues", {}).get("pageInfo", {}).get("hasNextPage", False)
            cursor = r.get("data", {}).get("issues", {}).get("pageInfo", {}).get("endCursor", None)

        while has_next_page and cursor:
            pagination_arguments = f"(first: {issues_per_page}, after: \"{cursor}\")"
            query = _generate_query_issues_with_pagination(pagination_arguments)
            response = self.graphql_query(query)

            page_num += 1
            scanned_issues = page_num * issues_per_page
            self.logging.info(
                f"Total Linear issues scanned: [{scanned_issues}]. Page number: [{page_num}]. Paginating with cursor: [{cursor}]")

            has_next_page = False
            cursor = None

            if response.status_code == 200:
                r = response.json()
                yield r
                has_next_page = r.get("data", {}).get("issues", {}).get("pageInfo", {}).get("hasNextPage", False)
                cursor = r.get("data", {}).get("issues", {}).get("pageInfo", {}).get("endCursor", None)

        return {}

    def get_all_issues_and_comments(self):
        result = {"data": {"issues": {"edges": [], "pageInfo": {}}}}
        issues_per_page = 100
        for r in self.get_issues_and_comments(issues_per_page):
            if r:
                result["data"]["issues"]["edges"] += r.get("data", {}).get("issues", {}).get("edges", [])
        return result



import logging


class HollowController:
    def __init__(self):
        self.client = None
        self.log_message_callback = None

    def set_config(self, config):
        return self.is_connected()

    def set_log_message_callback(self, log_message_callback):
        self.log_message_callback = log_message_callback

    def get_name(self):
        return "Hollow"

    def is_connected(self):
        return False

    def get_data(self, include_coments=False, limit=None):
        return {}

    def post_comment(self, issue, comment):
        return self.is_connected()

    def log_message(self, message, level=logging.INFO):
        if self.log_message_callback:
            self.log_message_callback(message, level)
        else:
            print(message)

    def pack_data(self, title, description, comments, url, ticket_key):
        ticket_data = {
            "ticket": {
                "title": {
                    "name": "title",
                    "data": title,
                    "data_type": "str"
                },
                "description": {
                    "name": "description",
                    "data": description,
                    "data_type": "str"
                },
                "comments": {
                    "name": "comments",
                    "data": comments,
                    "data_type": "list"
                }
            },
            "url": url,
            "issue_id": ticket_key
        }
        return ticket_data



import json
import logging
from WrikePy import *


try:
    from . import hollow_controller as hollow_controller
except Exception:
    import n0s1.controllers.hollow_controller as hollow_controller

class WrikeController(hollow_controller.HollowController):
    def __init__(self):
        super().__init__()
        self._client = None

    def set_config(self, config):
        TOKEN = config.get("token", "")
        base_url = "https://www.wrike.com/api/v4"
        self._client = Wrike(base_url, TOKEN)
        return self.is_connected()

    def get_name(self):
        return "Wrike"

    def is_connected(self):
        if self._client:
            acc = Account(self._client)
            response = acc.query__account()
            logged_in = False
            try:
                if 200 <= response.status_code < 300:
                    acc_info = json.loads(response.text)
                    logged_in = True
                    self.log_message(f"Logged to {self.get_name()} as {acc_info}")
            except Exception as ex:
                self.log_message(str(ex), logging.WARNING)
                pass
            if not logged_in:
                self.log_message(f"Unable to connect to {self.get_name()} instance. Check your credentials.", logging.ERROR)
                return False

            t = Tasks(self._client, parameters={"fields": ["description"]})
            response = t.query__tasks()
            list_tasks = False

            try:
                if 200 <= response.status_code < 300:
                    tasks = json.loads(response.text)
                    task_list = tasks.get("data", [])
                    if len(task_list) > 0:
                        list_tasks = True
                        return True
            except Exception as ex:
                self.log_message(str(ex), logging.WARNING)
                pass
            if not list_tasks:
                self.log_message(f"Unable to list {self.get_name()} tasks. Check your permissions.", logging.ERROR)
        return False

    def get_data(self, include_coments=False, limit=None):
        if not self._client:
            return {}

        t = Tasks(self._client, parameters={"fields": ["description"]})
        response = t.query__tasks()
        tasks = {}
        try:
            if 200 <= response.status_code < 300:
                tasks = json.loads(response.text)
        except Exception as ex:
            self.log_message(str(ex), logging.WARNING)

        task_list = tasks.get("data", [])
        for t in task_list:
            title = t.get("title", "")
            description = t.get("description", "")
            url = t.get("permalink", "")
            comments = []
            if task_id := t.get("id", None):
                if include_coments:
                    comments_obj = Comments(self._client, [task_id])
                    response = comments_obj.query__tasks_taskId_comments()
                    json_data = {}
                    try:
                        if 200 <= response.status_code < 300:
                            json_data = json.loads(response.text)
                    except Exception as ex:
                        self.log_message(str(ex), logging.WARNING)

                    c_data = json_data.get("data", [])
                    for c in c_data:
                        comments.append(c.get("text", ""))
                ticket = self.pack_data(title, description, comments, url, task_id)
                yield ticket

    def post_comment(self, task_id, comment):
        if not self._client:
            return False
        comment = comment.replace("<REDACTED>", "**********")
        comment = comment.replace("\n", "<br>")
        comments_obj = Comments(self._client, [task_id], parameters={"text": comment, "plainText": False})
        if comments_obj:
            response = comments_obj.create__tasks_taskId_comments()
            try:
                if 200 <= response.status_code < 300:
                    json_data = json.loads(response.text)
                    added_comment = json_data.get("data", [])[0]
                    if len(added_comment) > 0:
                        return True
            except Exception as ex:
                self.log_message(str(ex), logging.WARNING)
        return False



import logging
import socket

try:
    import clients.http_client as http_client
except Exception:
    import n0s1.clients.http_client as http_client


def _get_local_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(("10.255.255.255", 1))
        local_ip = s.getsockname()[0]
    except:
        local_ip = "127.0.0.1"
    finally:
        s.close()
    return local_ip


class Spark1(http_client.HttpClient):
    def __init__(self, headers: dict = None, server: str = None, options: dict[str, str] = None,
                 basic_auth: tuple[str, str] = None, token_auth: str = None, validate=False,
                 get_server_info: bool = True, async_: bool = False, async_workers: int = 5,
                 max_retries: int = 3, timeout: int = None,
                 auth: tuple[str, str] = None):
        self.base_url = "https://api.spark1.us"
        # self.base_url = "http://127.0.0.1:5000"
        self.local_ip = _get_local_ip()
        if server:
            self.base_url = server
        authorization = basic_auth
        if token_auth:
            authorization = token_auth
        if not headers:
            headers = {
                "Content-Type": "application/json",
                "Authorization": authorization,
            }
        else:
            headers["Authorization"] = authorization
        super().__init__(uri=self.base_url, logging=logging, headers=headers)

    def is_connected(self, config=None):
        if config is None:
            config = {}
        data = {
            "scanner_ip": self.local_ip,
            "scan_target": config.get("scan_target", ""),
            "report_format": config.get("report_format", "")
        }
        auth_url = self.base_url + "/api/v1/auth"
        try:
            r = self._post_request(auth_url, json=data)
            if r.status_code == 200:
                session_token = r.json().get("token", "")
                if session_token and len(session_token) > 0:
                    return True
        except Exception as ex:
            logging.info(str(ex))
            return False
        return False




import datetime
import logging
import re
import time

try:
    from . import hollow_controller as hollow_controller
except Exception:
    import n0s1.controllers.hollow_controller as hollow_controller


class SlackController(hollow_controller.HollowController):
    def __init__(self):
        super().__init__()
        self._client = None

    def set_config(self, config):
        from slack_sdk import WebClient
        from slack_sdk.errors import SlackApiError
        TOKEN = config.get("token", "")
        self._client = WebClient(token=TOKEN)
        return self.is_connected()

    def get_name(self):
        return "Slack"

    def is_connected(self):
        if user := self._client.auth_test():
            self.log_message(f"Logged to Slack as {user}")
        else:
            self.log_message(f"Unable to connect to Slack. Check your credentials.", logging.ERROR)
            return False
        return True

    def get_data(self, include_coments=False, limit=None):
        max_day_range = 365 * 100
        range_days = 1
        now = datetime.datetime.now()

        # Slack query by timestamp works like "greater than >" and "less than <" operators as opposed to ">=" and "<=".
        # If you want to pull messages from 2024-07-14 you have to provide the following query: after:2024-07-13 before:2024-07-15
        # Notice that the messages from the starting date (after:2024-07-13) and the end date (before:2024-07-15) are not included to the query results
        end_day = now + datetime.timedelta(days=1)
        start_day = now - datetime.timedelta(days=range_days)

        start_day_str = start_day.strftime("%Y-%m-%d")
        end_day_str = end_day.strftime("%Y-%m-%d")

        query = f"after:{start_day_str} before:{end_day_str}"
        days_counter = 0
        while days_counter < max_day_range:
            messages = self.run_slack_query(query)
            for m in messages:
                len_messages = len(m)
                if len_messages <= 0:
                    range_days = range_days * 2
                else:
                    range_days = 1
                    for item in m:
                        message = item.get("text", "")
                        iid = item.get("iid", "")
                        url = item.get("permalink", "")
                        ticket = self.pack_data(message, item, url, iid)
                        yield ticket

            end_day = start_day + datetime.timedelta(days=1)
            start_day = start_day - datetime.timedelta(days=range_days)
            start_day_str = start_day.strftime("%Y-%m-%d")
            end_day_str = end_day.strftime("%Y-%m-%d")
            query = f"after:{start_day_str} before:{end_day_str}"
            days_counter += range_days

    def post_comment(self, issue, comment):
        from slack_sdk.errors import SlackApiError
        try:
            channel_id, thread_ts = self.extract_channel_id_and_ts(issue)
            if comment and len(comment) > 0 and len(channel_id) > 0 and len(thread_ts) > 0:
                response = self._client.chat_postMessage(
                    channel=channel_id,
                    text=comment,
                    thread_ts=thread_ts,
                    unfurl_links=False
                )

                self.log_message(f"Message sent successfully")
                response_ts = response.get("ts", "")
                self.log_message(f"Thread Timestamp: {response_ts}")

        except SlackApiError as e:
            error_message = e.response.get("error", "")
            self.log_message(f"Error sending message: {error_message}", logging.ERROR)

    def pack_data(self, message, raw_data, url, iid):
        channel_id = raw_data.get("channel", {}).get("id", "")
        channel_name = raw_data.get("channel", {}).get("name", "")
        is_channel = raw_data.get("channel", {}).get("is_channel", "")
        timestamp = raw_data.get("ts", "")
        slack_type = raw_data.get("type", "")
        ticket_data = {
            "ticket": {
                "message": {
                    "name": "message",
                    "data": message,
                    "data_type": "str"
                },
            },
            "url": url,
            "issue_id": url,
            "raw_data": {
                "iid": iid,
                "channel_name": channel_name,
                "channel_id": channel_id,
                "is_channel": is_channel,
                "timestamp": timestamp,
                "slack_type": slack_type
            }
        }
        return ticket_data

    def search_with_rate_limit(self, query, sort, cursor):
        from slack_sdk.errors import SlackApiError
        response = None
        try:
            response = self._client.search_messages(query=query, sort=sort, cursor=cursor)
        except SlackApiError as ex:
            message = str(ex) + f" client.search_messages()"
            self.log_message(message, logging.WARNING)
            retry_after = ex.response.headers.get("Retry-After", "")
            if len(retry_after) <= 0:
                retry_after = ex.response.headers.get("retry-after", "")
            if len(retry_after) > 0:
                retry_after = int(retry_after)
            else:
                retry_after = 30
            retry_after += 5
            self.log_message(f"Rate limit reached! Retrying after [{retry_after}] seconds...", logging.WARNING)
            time.sleep(retry_after)
            response = self.search_with_rate_limit(query, sort, cursor)

        except Exception as ex:
            message = str(ex) + f" client.search_messages()"
            self.log_message(message, logging.ERROR)

        return response

    def run_slack_query(self, query):
        cursor = ""
        self.log_message(f"Scanning Slack messages: [{query}]...")
        time.sleep(0.2)
        if response := self.search_with_rate_limit(query=query, sort="timestamp", cursor="*"):
            messages = response.get("messages", {}).get("matches", [])
            cursor = response.get("messages", {}).get("pagination", {}).get("next_cursor", "")
            yield messages

        while len(cursor) > 0:
            cursor = ""
            time.sleep(0.2)
            if response := self.search_with_rate_limit(query=query, sort="timestamp", cursor=cursor):
                messages = response.get("messages", {}).get("matches", [])
                cursor = response.get("messages", {}).get("pagination", {}).get("next_cursor", "")
                yield messages

    def extract_channel_id_and_ts(self, link):
        # Extract the channel ID and message timestamp from the link
        match = re.search(r'archives/([^/]+)/p(\d+)', link)
        if match:
            channel_id = match.group(1)
            message_ts = f"{match.group(2)[:10]}.{match.group(2)[10:]}"
            return channel_id, message_ts
        else:
            self.log_message("Invalid Slack link format", logging.ERROR)

        return "", ""



"""A setuptools based setup module.

See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject
"""

import pathlib
import re

# Always prefer setuptools over distutils
from setuptools import setup, find_packages

here = pathlib.Path(__file__).parent.resolve()  # current path
long_description = (here / "README.md").read_text(encoding="utf-8")
with open(here / "requirements.txt") as fp:
    install_reqs = [r.rstrip() for r in fp.readlines() if not r.startswith("#")]


def get_version():
    file = here / "src/n0s1/__init__.py"
    return re.search(
        r'^__version__ = [\'"]([^\'"]*)[\'"]', file.read_text(), re.M
    )[1]


setup(
    name="n0s1",
    version=get_version(),
    description="Secret Scanner for Slack, Jira, Confluence, Asana, Wrike and Linear. Prevent credential leaks with n0s1.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://spark1.us/n0s1",
    author="Spark 1",
    author_email="contact@spark1.us",
    classifiers=["Development Status :: 3 - Alpha",
                 "Intended Audience :: Developers",
                 "Intended Audience :: Information Technology",
                 "Operating System :: OS Independent",
                 "Topic :: Security",
                 "Topic :: Software Development",
                 "Topic :: System :: Monitoring",
                 "Topic :: Utilities",
                 "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
                 "Programming Language :: Python :: 3.8",
                 "Programming Language :: Python :: 3.9",
                 "Programming Language :: Python :: 3.10",
                 "Programming Language :: Python :: 3.11",
                 ],  # Classifiers help users find your project by categorizing it https://pypi.org/classifiers/
    keywords="security, cybersecurity, scanner, secret scanner, secret leak, data leak, Slack, Jira, Confluence, Asana, Wrike, Linear, security scanner",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=3.8, <4",

    # For an analysis of "install_requires" vs pip's requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=install_reqs,

    # List additional groups of dependencies here (e.g. development
    # dependencies). Users will be able to install these using the "extras"
    # syntax, for example: $ pip install sampleproject[dev]
    # Similar to `install_requires` above, these must be valid existing projects
    extras_require={"dev": ["check-manifest"],
                    "test": ["coverage"],
                    },

    include_package_data=True,
    package_data={"n0s1": ["src/n0s1/config/*"],
                  },

    # The following would provide a command called `n0s1` which
    # executes the function `main` from this package when invoked:
    entry_points={"console_scripts": ["n0s1=n0s1.n0s1:main", ],
                  },

    project_urls={"Bug Reports": "https://github.com/spark1security/n0s1/issues",
                  "Funding": "https://gofund.me/c6a0520c",
                  "Source": "https://github.com/spark1security/n0s1",
                  },
)
